@@ -14,7 +14,7 @@ from .chatglm import ChatGLMForConditionalGeneration, ChatGLMTokenizer
 
 
 class CastOutputToFloat(nn.Sequential):
-    def forward(self, x): 
+    def forward(self, x):
         return super().forward(x).to(torch.float32)
 
 
@@ -29,16 +29,18 @@ class GlmLora:
         lora_dropout: float = 0.1,
         load_in_8bit: bool = False,
         infer_mode: bool = False,
+        cast_small_parameters: bool = True,
     ):
         print(f"Loading tokenizer and model of {model_id}")
         self.infer_mode = infer_mode
         self.load_in_8bit = load_in_8bit
+        self.cast_small_parameters = cast_small_parameters
         if self.load_in_8bit:
             import bitsandbytes as bnb
             self.device = None
             model = self._load_8bit_glm(model_id)
         else:
-            self.device = device
+            self.device = device or "cuda"
             model = self._load_glm(model_id)
         self.tokenizer = ChatGLMTokenizer.from_pretrained(model_id)
         self.config = LoraConfig(
@@ -61,22 +63,24 @@ class GlmLora:
     def __cast_to(self, model: nn.Module, dtype: torch.Type) -> nn.Module:
         for param in model.parameters():
             param.requires_grad = False
-            if param.ndim == 1:
+            if param.ndim == 1 and param.dtype != dtype:
                 # cast the small parameters (e.g. layernorm, bias) to fp32 for stability
                 param.data = param.data.to(dtype)
         return model
-    
+
     def _load_8bit_glm(self, model_id: str) -> PreTrainedModel:
         model = ChatGLMForConditionalGeneration.from_pretrained(
             model_id, load_in_8bit=True, device_map="auto")
-        if self.infer_mode:
-            dtype = torch.float16
-        else:
-            dtype = torch.float32
-        model = self.__cast_to(model, dtype)
-        model.lm_head = CastOutputToFloat(model.lm_head)
+        if self.cast_small_parameters:
+            if self.infer_mode:
+                dtype = torch.float16
+                model = self.__cast_to(model, dtype)
+            else:
+                dtype = torch.float32
+                model = self.__cast_to(model, dtype)
+                model.lm_head = CastOutputToFloat(model.lm_head)
         return model
-    
+
     def _load_glm(self, model_id: str) -> PreTrainedModel:
         model = ChatGLMForConditionalGeneration.from_pretrained(
             model_id).to(self.device)
@@ -100,7 +104,7 @@ class GlmLora:
         lr: float = 2e-4,
         num_epochs: int = 10,
         warmup_steps: Optional[int] = None,
-        accumulate_steps: Optional[int] = 8,
+        accumulate_steps: Optional[int] = 32,
         out_dir: str = "./output/",
         print_every: int = 10,
     ):
@@ -110,7 +114,7 @@ class GlmLora:
         if `warmup_steps` is None, will use one epoch to warmup by default
         if `accumulate_steps` is None, will use 1 as default
         """
-        # turnoff when infer
+        # turn on when infer
         self.model.config.use_cache = False
         if self.device is not None:
             self.model.to(self.device).train()
@@ -139,6 +143,7 @@ class GlmLora:
         if quant_bit is not None:
             self.model.quantize(quant_bit)
 
+        # 8bit do not use device, device is None
         if self.device is not None:
             self.model.to(self.device).eval()
         else:
@@ -149,7 +154,7 @@ class GlmLora:
             history = []
         res = []
         for response, history in self.model.stream_chat(
-            self.tokenizer, inp, history, max_len):
+                self.tokenizer, inp, history, max_len):
             query, response = history[-1]
             res.append(response)
         answer = "".join(response)
